;ͻ
;                                                                          
;                    Copyright (c) 1995 Aldo Ferraro                       
;                          All Rights Reserved                             
;                                                                          
;Ķ
;                                                                          
;  Filename      :  PX.ASM                          Version : 1.0          
;                                                                          
;  Program Title :  Project X                                              
;                                                                          
;  Description   :  Demo program to get & set the video mode using the     
;                   VESA VBE or NON-VESA VBE calls.                        
;                                                                          
;  Author Name   :  Aldo Ferraro                                           
;                                                                          
;  Language      :  Turbo Assembler version 3.1 using MASM Mode            
;                                                                          
;  Command Line  :  TASM  PX.ASM /m2 /q                                    
;                   TLINK PX.OBJ VESA.OBJ DAC.OBJ /3 /P /x /yx             
;                                                                          
;ͼ

	%TITLE	"Project X"
	PAGE	58, 132

;ͻ
;                   Set memory model and instruction set                   
;ͼ

	.MODEL 	large
	.386P
	.387

;ͻ
;                              Stack Segment                               
;ͼ

	.STACK

;ͻ
;                             External Names                               
;ͼ


	EXTRN	get_video_mode:PROC		; get current video mode
	EXTRN	set_video_mode:PROC             ; set required video mode
	EXTRN	clr_dac:PROC			; clear DAC RGB colour block
	EXTRN	set_dac:PROC			; set DAC RGB colour

	EXTRN 	original_mode:WORD              ; save original video mode
	EXTRN 	vbe_mode:WORD                   ; VBE video mode request


;ͻ
;                            Public Procedures                             
;ͼ

	PUBLIC	cls_13				; cls mode 19 chain-4 display
	PUBLIC	bail_out			; exit to DOS with error msg

;ͻ
;                                 Macros                                   
;ͼ

show_mouse	macro				; show mouse cursor
	mov	ax,0001h
	int	33h
		endm

hide_mouse	macro				; hide mouse cursor
	mov	ax,0002h
	int	33h
		endm

;ͻ
;                                 Equates                                  
;ͼ


;ͻ
;                                Data Segment                              
;ͼ

		.DATA

;

copyright	db	"Copyright (c) 1995 Aldo Ferraro"
bye		db	"Have a nice day!$"

screen_buffer	dd	0A0000000h		; screen segment:offset

;Ŀ
;                              Error Messages                              
;

vga_error	db	"VGA Colour Detection Failed$"
mem_error	db	"Memory Allocation Failure$"

;ͻ
;                               Code Segment                               
;ͼ

	.CODE

;Ŀ
;                                 M A I N                                  
;

main		proc

;Ŀ
;                           Code Initialization                            
;

	.STARTUP

	call	get_video_mode			; get current video mode

;Ŀ
; Check to confirm the current computer system is using an active  VGA  or 
; SVGA adaptor card with a colour monitor. If VGA detection fails,  or the 
; current computer system is not using a colour display monitor, terminate 
; program execution with an error message.                                 
;

	mov     ax,1A00h         		; bios function 1Ah, sub 00h
	int     10h                     	; display combination code
	cmp     al,1Ah           		; is a VGA card installed ?
	jne     bail_out            		; no, bail out
	cmp     bl,08h           		; is monitor colour ?
	je      vga_found           		; no, bail out
	mov	dx,offset ds:vga_error		; dx -> error message
	jmp	bail_out

vga_found:

;Ŀ
;                         Set required video mode                          
;

	call	set_video_mode			; set desired video mode

;----------------------------------------------------------------------------

	mov	eax,25344501h			; re-define colour 01h
	call	set_dac
	mov	al,01h				; set colour request
	call	cls_13             		; clear screen using colour
	show_mouse				; display mouse cursor

;----------------------------------------------------------------------------
	mov	ah,07				; wait for key press
	int	21h
;----------------------------------------------------------------------------

;Ŀ
; If a critical error occurred, clean up and display the appropriate error 
; message on exit, else clean up and display a bye message on exit.        
;

	mov	dx,offset ds:bye		; dx -> bye message

bail_out:
	mov	ax,original_mode		; get original video mode
	mov	vbe_mode,ax             	; vbe_mode = original mode
	call	set_video_mode          	; restore original mode
	mov	ah,9				; display error message
	int	21h

	.EXIT					; exit to DOS

main		endp

;ͻ
;                                                                          
;  *>   P R O C E D U R E S   <-*  
;                                                                          
;ͼ

;ͻ
;                   Clear Video Mode 13h Chain-4 Display                   
;Ķ
;                                                                          
; Description :  Clear chain-4 VGA 320x200x256 display using the requested 
;                colour.                                                   
;                                                                          
; On Entry    :  al = requested colour                                     
;                                                                          
; Returns     :  none                                                      
;                                                                          
;ͼ

cls_13   	proc

	les	di,screen_buffer                ; es:di -> screen buffer
	mov    	cx,320/4*200        		; get screen pixel count
	mov    	ah,al               		; colour request now in ax
	push   	ax                  		; put colour request in eax
	shld   	eax,eax,16
	pop    	ax                  		; colour request now in eax
	rep    	stosd               		; fill all four bit planes
	ret

cls_13   	endp

;ͻ
;                            End of Assembly                               
;ͼ

	END

