;ͻ
;                                                                          
;                    Copyright (c) 1995 Aldo Ferraro                       
;                          All Rights Reserved                             
;                                                                          
;Ķ
;                                                                          
;  Filename      :  VESA.ASM                        Version : 1.0          
;                                                                          
;  Program Title :  VGA & VESA VBE Video Procedures                        
;                                                                          
;  Description   :  VGA BIOS & VESA VGA BIOS Extension Video Procedures    
;                                                                          
;  Author Name   :  Aldo Ferraro                                           
;                                                                          
;  Language      :  Turbo Assembler version 3.1 using MASM Mode            
;                                                                          
;  Command Line  :  TASM  VESA.ASM /m2 /q                                  
;                                                                          
;ͼ

	%TITLE	"VGA & VESA VBE Video Procedures"
	PAGE	58, 132

;ͻ
;                   Set memory model and instruction set                   
;ͼ

	.MODEL 	large
	.386P
	.387

;ͻ
;                              Public Names                                
;ͼ

	PUBLIC	original_mode
	PUBLIC	vbe_mode

;ͻ
;                            Public Procedures                             
;ͼ

	PUBLIC	set_video_mode			; set specific video mode
	PUBLIC	get_video_mode			; get current video mode

;ͻ
;                                 Macros                                   
;ͼ

;ͻ
;                                 Equates                                  
;ͼ


;ͻ
;                                Data Segment                              
;ͼ

		.DATA

original_mode	dw	0003h			; save original video mode
vga_mode	dw      0013h			; VGA video mode
vbe_mode        dw	0013h			; VBE video mode

;Ŀ
;                              Error Messages                              
;


;Ŀ
;                       VBE Controller Information                         
;

vbe_signature			db	'    '		; 'VESA'
vbe_version			dw	?
oem_string_pointer		dw	?
				dw	?
capabilities			dd	?		; version 1.2
video_mode_pointer		dw	?
				dw	?
total_video_memory		dw	?		; version 1.2
oem_software_revision		dw	?		; version 2.0
oem_vendor_name_pointer		dw	?		; version 2.0
				dw	?
oem_product_revision_pointer	dw	?		; version 2.0
				dw	?
reserved			db	222 dup(0)
oem_data			db	256 dup(0)	; version 2.0

;Ŀ
;                           VBE Mode Information                           
;

; -> Supported by all VBE versions...

mode_A_attributes		dw	?
window_A_attributes 		db	?
window_B_attributes 		db	?
window_granularity		dw	?
window_size			dw	?
window_A_segment		dw	?
window_B_segment		dw	?
window_function_pointer		dd 	?
bytes_per_scan_line		dw	?

; -> Extended data fields defined in VBE versions 1.2 and above...

x_resolution			dw	?
y_resolution			dw	?
x_character_size		db	?
y_character_size		db	?
number_of_planes		db	?
bits_per_pixel			db	?
number_of_banks			db	?
memory_model			db	?
bank_size			db	?
reserved_1			db	?

;-> Direct colour modes defined in VBE versions 1.2 and above...

red_mask_size			db	?
red_field_position		db	?
green_mask_size			db	?
green_field_position		db	?
blue_mask_size			db	?
blue_field_position		db	?
reserved_mask_size		db	?
reserved_field_position		db	?
direct_color_mode_info		db	?

; -> Supported in VBE versions 2.0 and above...

physical_base_pointer		dd 	0
offscreen_memory_offset		dd 	0
offscreen_memory_size		dd 	0
reserved_2			db	206 dup(0)

;ͻ
;                               Code Segment                               
;ͼ

	.CODE

;ͻ
;                         Get Current Video Mode                           
;Ķ
;                                                                          
; Description :  Detect if VBE video modes are available. On success, save 
;                VBE Controller Information. If VBE not available, get the 
;                current VGA video mode.                                   
;                                                                          
; On Entry    :  none                                                      
;                                                                          
; Returns     :  none                                                      
;                                                                          
;ͼ

get_video_mode	proc

	mov	ax,4F00h			; use VBE sub-service 0
	lea	di,vbe_signature		; es:di -> VBE 512-byte buf
	push	ds				; es -> ds
	pop	es
	int	10h				; get VBE controller info
	cmp	ax,004Fh			; was VBE call successful ?
	jne	not_vbe				; no, get current VGA mode
	cmp	dword ptr [vbe_signature],'ASEV'; is signature OK ?
	jne	not_vbe                 	; no, get current VGA mode

;Ŀ
;                     Get and save current video mode                      
;

	mov	ax,4F03h			; use VBE sub-service 3
	int	10h				; get current VBE video mode
	cmp	ax,004Fh			; was VBE call successful ?
	jne	not_vbe				; no, get current VGA video mode
	xchg	ax,bx				; put VBE video mode in ax
	jmp     got_mode                	; got VBE video mode

not_vbe:
	mov	ah,0Fh				; use BIOS service 0Fh
	int	10h				; get current video mode

got_mode:
	mov	original_mode,ax		; save original video mode
	ret

get_video_mode	endp

;ͻ
;                         Set Desired Video Mode                           
;Ķ
;                                                                          
; Description :  Set requested VBE SVGA video mode. If VBE unavailable, or 
;                detection of the VBE fails, set alternate video mode.  On 
;                successfully setting VBE, get information on video mode.  
;                                                                          
; On Entry    :  vbe_mode = VESA VBE SVGA video mode                       
;                vga_mode = alternate VGA video mode                       
;                                                                          
; Exception   :  If the requested vbe_mode <= 13h,  the video mode will be 
;                set using a NON-VBE service call.                         
;                                                                          
; Warning     :  If vbe_mode <= 13h, vga_mode will equal vbe_mode.  Hence, 
;                vga_mode will contain a new alternate VGA video mode.     
;                                                                          
; Returns     :  none                                                      
;                                                                          
; Note        :  Modes after 10Ch were introduced in VBE v1.2.  Hence,  in 
;                selecting these modes, further error checking	is required 
;                to ensure these modes are supported by the installed VBE. 
;                                                                          
;ͼ

set_video_mode	proc

	mov	bx,vbe_mode			; get the requested VBE mode
	cmp	bx,0013h			; was mode <= 19 requested ?
	jle	use_vga				; yes, set as VGA mode
	mov	ax,4F02h			; no, use VBE sub-service 2
	int	10h				; set requested VBE mode
	cmp	ax,004Fh			; was VBE call successful ?
	jne	no_vbe  			; no, set alternate VGA mode
	cmp	dword ptr [vbe_signature],'ASEV'; is signature OK ?
	jne	no_vbe                  	; no, set alternate VGA mode

;Ŀ
; If the  VBE  video mode was successfully set, get the characteristics of 
; the current VBE video mode, and save the VBE mode information.           
;

	mov	ax,4F01h			; use VBE sub-service 1
	mov	cx,vbe_mode			; get active VBE video mode
	lea	di,mode_A_attributes		; es:di -> VBE 256-byte buf
	push	ds				; es -> ds
	pop	es
	int	10h				; save VBE mode information
	cmp	ax,004Fh			; was VBE call successful ?
	jne	no_vbe  			; no, set alternate VGA mode
	jmp	mode_set			; return to caller

use_vga:
	xchg	bx,vga_mode			; use VBE request in VGA call

no_vbe:
	mov	ax,vga_mode			; get alternate VGA mode
	int	10h                     	; set alternate VGA mode

mode_set:
	ret

set_video_mode	endp

;ͻ
;                            End of Assembly                               
;ͼ

	END

